<?php

namespace Drupal\select_text_value;

use Drupal\Core\Render\Element;
use Drupal\Core\StringTranslation\StringTranslationTrait;

class WidgetHelper {

  use StringTranslationTrait;

  /**
   * Simple array of default values for all form widgets
   *
   * @return array
   */
  public static function defaultSettings() {
    return [
      'select_type' => 'select',
      'allowed_values' => '',
      'custom_value_label' => 'Other',
    ];
  }

  /**
   * @param array $form
   * @param array $settings
   *
   * @return array
   */
  public function settingsForm(array $form, array $settings) {

    $form['select_type'] = [
      '#title' => $this->t('Select type'),
      '#type' => 'select',
      '#options' => [
        'select' => $this->t('Select'),
        'radios' => $this->t('Radio Buttons'),
      ],
      '#default_value' => $settings['select_type'],
      '#required' => TRUE,
    ];

    $form['custom_value_label'] = [
      '#title' => $this->t('Custom value label'),
      '#type' => 'textfield',
      '#default_value' => $settings['custom_value_label'],
      '#required' => TRUE,
    ];

    $form['allowed_values'] = [
      '#title' => $this->t('Allowed values list'),
      '#type' => 'textarea',
      '#description' => $this->allowedValuesDescription(),
      '#default_value' => $settings['allowed_values'],
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * @param array $settings
   *
   * @return array
   */
  public function settingsSummary(array $settings) {
    $summary = [];
    $summary[] = $this->t('Select type').': '.$settings['select_type'];
    $summary[] = $this->t('Custom value label').': '.$settings['custom_value_label'];
    $summary[] = $this->t('Allowed values') . ': ';

    foreach (explode("\n", $settings['allowed_values']) as $value) {
      $summary[] = $value;
    }

    return $summary;
  }

  /**
   * @param array $settings
   * @param $required
   *
   * @return array
   */
  public function createSelectField(array $settings, $required) {
    $empty_option = $required ? $this->t('Select') : $this->t('None');

    return [
      '#type' => $settings['select_type'],
      '#required' => $required,
      '#empty_option' => "- {$empty_option} -",
      '#options' =>
        $this->allowedValuesToArray($settings['allowed_values']) +
        ['_custom_value' => $settings['custom_value_label']],
    ];
  }

  /**
   * Move the original form element title to the select field
   *
   * @param array $element
   *
   * @return array
   */
  public function setFormElementTitle(array $element) {
    $element['select']['#title'] = $element['field']['#title'];
    unset($element['field']['#title']);
    return $element;
  }

  /**
   * Set the default values when the form element is loaded
   *
   * @param $items
   * @param $delta
   * @param $element
   * @param $settings
   *
   * @return mixed
   */
  public function setFormElementDefaultValues($items, $delta, $element, $settings) {
    $allowed_values = $this->allowedValuesToArray($settings['allowed_values']);

    if (!empty($items[$delta]->value)) {
      $element['select']['#default_value'] = '_custom_value';

      if (in_array($items[$delta]->value, $allowed_values)) {
        $element['select']['#default_value'] = $items[$delta]->value;
        $element['field']['#default_value'] = '';
      }
    }

    return $element;
  }

  /**
   * Convert the stored allowed_values like string into an array
   *
   * @param $string
   * @return array
   */
  public function allowedValuesToArray($string) {
    $allowed_values = [];
    $lines = array_filter(explode("\n", $string));

    foreach ($lines as $line) {
      $line = trim($line);
      if (strpos($line, '|') !== FALSE){
        list($key, $value) = explode('|', $line);
        $allowed_values[ trim($key) ] = trim($value);
      }
      else {
        $allowed_values[ $line ] = $line;
      }
    }

    return $allowed_values;
  }

  /**
   * Allowed values list field description
   */
  public function allowedValuesDescription() {
    $description = '<p>' . t('The possible values this field can contain. Enter one value per line, in the format key|label.');
    $description .= '<br/>' . t('The key is the stored value, and must be numeric. The label will be used in displayed values and edit forms.');
    $description .= '<br/>' . t('The label is optional: if a line contains a single number, it will be used as key and label.');
    $description .= '<br/>' . t('Lists of labels are also accepted (one label per line), only if the field does not hold any values yet. Numeric keys will be automatically generated from the positions in the list.');
    $description .= '</p>';
    return $description;
  }

  /**
   * String fields only need to hide the value field itself
   *
   * @param $element
   *
   * @return mixed
   */
  public static function stringFieldAssignStates($element) {
    foreach (Element::children($element) as $delta){
      if (!empty($element[$delta]['select'])){
        $selector = $element[$delta]['select']['#name'];
        $element[$delta]['field']['#states'] = [
          'visible' => [
            ":input[name='{$selector}']" => ['value' => '_custom_value']
          ]
        ];
      }
    }
    return $element;
  }

  /**
   * Formatted text fields need to hide both the text area and the Format select
   *
   * @param $element
   *
   * @return mixed
   */
  public static function formattedTextFieldAssignStates($element) {
    foreach (Element::children($element) as $delta){
      if (!empty($element[$delta]['select'])){
        $selector = $element[$delta]['select']['#name'];
        $element[$delta]['field']['value']['#states'] = [
          'visible' => [
            ":input[name='{$selector}']" => ['value' => '_custom_value']
          ]
        ];
        $element[$delta]['field']['format']['#states'] = [
          'visible' => [
            ":input[name='{$selector}']" => ['value' => '_custom_value']
          ]
        ];
      }
    }
    return $element;
  }

  /**
   * "Massage" the user input values into place for storage
   *
   * @param array $values
   *
   * @return array
   */
  public function stringFieldMassageFormValues(array $values) {
    $new_values = [];

    foreach ($values as $value) {
      $new_value = [
        'value' => ($value['select'] == '_custom_value') ? $value['field'] : $value['select'],
        '_original_delta' => $value['_original_delta'],
      ];

      $new_values[] = $new_value;
    }
    return $new_values;
  }

  /**
   * "Massage" the user input values into place for storage
   *
   * @param array $values
   *
   * @return array
   */
  public function formattedTextFieldMassageFormValues(array $values) {
    $new_values = [];

    foreach ($values as $value) {
      $new_value = [
        'value' => ($value['select'] == '_custom_value') ? $value['field']['value'] : $value['select'],
        'format' => $value['field']['format'],
        '_original_delta' => $value['_original_delta'],
      ];

      $new_values[] = $new_value;
    }
    return $new_values;
  }
}
